local AddonName, AddonTable = ...

local Addon = AddonTable.Addon
local Module = Addon:NewModule("Broker")

local L = LibStub("AceLocale-3.0"):GetLocale(AddonName, false)

local LQT = LibStub("LibQTip-1.0")
local AT = LibStub("AceTimer-3.0")

local tooltip
local playerTarget
local timerHandle
local datasource = {
  type = "data source",
  label = AddonName,
}

local vigilanceId = 50720
local _, _, vigilanceIcon = GetSpellInfo(vigilanceId)

local function FormatExpirationTime(unit)
  local time = unit:GetExpirationTime()
  if time then
    local diff = time - GetTime()
    return format("%d:%02d", diff / 60, diff % 60)
  end
  return ""
end

-- Custom Cellprovider - adds coloring
local myProvider, cellPrototype, baseCellPrototype = LQT:CreateCellProvider(LQT.LabelProvider)

function cellPrototype:SetupCell(tooltip, value, justification, font, color, ...)
  -- Call the base method
  local width, height = baseCellPrototype.SetupCell(self, tooltip, value, justification, font)

  -- The base LabelProvider stores its FontString in the fontString attribute
  local fs = self.fontString

  -- Set color
  if color and type(color) == 'table' then
    fs:SetTextColor(color.r, color.g, color.b)
  end

  -- Return the dimensions
  return width, height
end

function cellPrototype:ReleaseCell()
  -- Revert to default color
  self.fontString:SetTextColor(1, 1, 1)
end

local titleColor = { r=1, g=0.82, b=0 }
local headerColor = { r=1, g=1, b=0.6 }

local function FillTooltip()
  local line
  tooltip:Clear()
  tooltip:AddHeader()
  tooltip:SetCell(1, 1, AddonName, "CENTER", 3, titleColor)
  tooltip:AddSeparator(5, 0, 0, 0, 0)
  if Module:IsEnabled() then
    tooltip:AddLine()
    tooltip:SetCell(3, 1, L["HEADER_OWNER"], headerColor)
    tooltip:SetCell(3, 2, L["HEADER_TARGET"], headerColor)
    tooltip:SetCell(3, 3, L["HEADER_TIME"], headerColor)

    for unit in Addon:IterateTargets() do
      tooltip:AddLine(unit:GetOwner().name, unit.name, FormatExpirationTime(unit))
    end
  else
    tooltip:SetCell(3, 1, L["DISABLED"], GameTooltipTextSmall, 3)
  end
end

local function OnEnter(button)
  tooltip = LQT:Acquire(AddonName, 3, "LEFT", "LEFT", "RIGHT")
  tooltip:SetDefaultProvider(myProvider)

  FillTooltip()

  tooltip:SmartAnchorTo(button)
  tooltip:Show()
end

local function OnLeave()
  LQT:Release(tooltip)
  tooltip = nil
end

local function OnClick()

end

local function UpdateText()
  datasource.text = format("%s %s", playerTarget.name, FormatExpirationTime(playerTarget))
end

local function BuffAdded(_, unit)
  if unit:GetOwner().GUID == UnitGUID("player") then
    playerTarget = unit
    UpdateText()
    timerHandle = AT:ScheduleRepeatingTimer(UpdateText, 1.0)
  end
end

local function BuffRemoved(_, unit)
  if unit == playerTarget then
    playerTarget = nil
    AT:CancelTimer(timerHandle)
    datasource.text = L["NOT_FOUND"]
  end
end

function Module:OnEnable()
  if datasource.text == nil then
    datasource = LibStub:GetLibrary("LibDataBroker-1.1"):NewDataObject(AddonName, datasource)
  end

  datasource.text = L["NOT_FOUND"]
  datasource.icon = vigilanceIcon
  datasource.OnEnter = OnEnter
  datasource.OnLeave = OnLeave
  datasource.OnClick = OnClick

  Addon.RegisterCallback(self, "AURA_GAINED", BuffAdded)
  Addon.RegisterCallback(self, "AURA_LOST", BuffRemoved)

  for unit in Addon:IterateTargets() do
    BuffAdded(nil, unit)
  end
end

function Module:OnDisable()
  datasource.text = ""
  datasource.icon = nil
  datasource.OnEnter = nil
  datasource.OnLeave = nil
  datasource.OnClick = nil

  Addon.UnregisterAllCallbacks(self)

  if playertarget ~= nil then
    BuffRemoved(nil, playerTarget)
  end
end
